// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KIMAP_VERSION_H
#define KIMAP_VERSION_H

#define KIMAP_VERSION_STRING "5.19.1"
#define KIMAP_VERSION_MAJOR 5
#define KIMAP_VERSION_MINOR 19
#define KIMAP_VERSION_PATCH 1
#define KIMAP_VERSION ((5<<16)|(19<<8)|(1))

#endif
